@@ -8,17 +8,20 @@ use Params::Validate qw(:all);
 use DBIx::DBSchema;
 use HTML::Template;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 my $init_params = 
 	{
+		strip_tablename			=> { type 	=> BOOLEAN,default 	=> 1},
+		begin_form				=> { type 	=> BOOLEAN,default 	=> 1},
+		end_form				=> { type 	=> BOOLEAN,default 	=> 1},
 	};
 
 sub new {
-	my $proto = shift;
-	my $class = ref($proto) || $proto;
-	my %opts  = validate(@_ , $init_params);
-	my $self  = {%opts};
+	my $proto 					= shift;
+	my $class 					= ref($proto) || $proto;
+	my %opts  					= validate(@_ , $init_params);
+	my $self  					= {%opts};
 	bless $self,$class;
 	return $self;
 }
@@ -47,18 +50,25 @@ sub html {
 	my $values		= $self->values;
 	foreach (@columns) {
     	my $col_schema = $tbl_schema->column($_);
+		my $name	= ($self->strip_tablename) ? '' : $self->tablename . '.';
+		$name		.= $col_schema->name;
     	my $length  = $col_schema->length;
     	$length = $length == 0 	? 10 	
 								: ($length > 50 ? 50 : $length) if ($length);
-    	my $field       = {     name => $col_schema->name ,
-								label => $col_schema->name,
+    	my $field       = {     name 	=> $name,
+								label	=> $col_schema->name,
 								pos		=> $column_pos,
-                            	value => $values->{$col_schema->name} ,
+                            	value => $values->{$col_schema->name}  
+											|| $col_schema->default,
                             	length => $length,
 								can_be_null => ($col_schema->null eq 'NULL'),
 								is_null => defined 
 											$values->{$col_schema->name}
 											? 0 : 1,
+								'row.pre' => defined $self->{cb_row_pre} 
+											? $self->{cb_row_pre}->call($col_schema->name) : '',
+								'row.post' => defined $self->{cb_row_post} 
+											? $self->{cb_row_post}->call($col_schema->name) : '',
                           };
 		# reimposto la label
 		if ($self->labels) {
@@ -81,7 +91,13 @@ sub html {
 	}
 	my $htmpl = $self->_new_html_template();
 	$htmpl->param(fields => \@fields);
-	$htmpl->param(hidden_fields => \@hidden_fields);
+	$htmpl->param('fields.hidden' => \@hidden_fields);
+	$htmpl->param('form.begin' => $self->begin_form);
+	$htmpl->param('form.end' => $self->end_form);
+	$htmpl->param('rows.pre' => defined $self->{cb_rows_pre} 
+								? $self->{cb_rows_pre}->call() : '');
+	$htmpl->param('rows.post' => defined $self->{cb_rows_post} 
+								? $self->{cb_rows_pre}->call() : '');
 	return $htmpl->output;
 }
 
@@ -90,10 +106,12 @@ sub _new_html_template() {
 	my $htmpl;
 	if ($self->tmpl_path) {
 		$htmpl = new HTML::Template( 	filename 	=> $self->tmpl_path,
-                               			vanguard_compatibility_mode=>1);
+                               			vanguard_compatibility_mode=>1,
+										global_vars => 1);
 	} else {
 		$htmpl = new HTML::Template(	scalarref => \$self->template,
-										vanguard_compatibility_mode=>1);
+										vanguard_compatibility_mode=>1,
+										global_vars=>1);
 	}
 	return $htmpl;
 	
@@ -183,40 +201,50 @@ sub _set_field_enums() {
 sub tmpl_path {
 	my $self 	= shift;
 	my @opt		= validate_pos(@_, {type => SCALAR | UNDEF, default => undef} );
-	return $opt[0] ?  $self->{tmpl_path} = $opt[0] : $self->{tmpl_path};
+	return defined $opt[0] ?  $self->{tmpl_path} = $opt[0] : $self->{tmpl_path};
 }
 
 
 sub dbh {
 	my $self 	= shift;
 	my @opt		= validate_pos(@_, {isa => 'DBI::db' , default => undef} );
-	return $opt[0] ?  $self->{dbh} = $opt[0] : $self->{dbh};
+	return defined $opt[0] ?  $self->{dbh} = $opt[0] : $self->{dbh};
 }
 
 sub tablename {
 	my $self 	= shift;
+	if (defined $self->tblschema && !defined $self->dbh) {
+		# prendo il nome direttamente dal tblschema
+		return $self->tblschema->name;
+	}
 	my @opt		= validate_pos(@_, {type => SCALAR, default => undef} );
-	return $opt[0] ?  $self->{tablename} = $opt[0] : $self->{tablename};
+	return defined $opt[0] ?  $self->{tablename} = $opt[0] : $self->{tablename};
+}
+
+sub strip_tablename {
+	my $self 	= shift;
+	my @opt		= validate_pos(@_, {type => BOOLEAN, default => undef} );
+	return defined $opt[0] ?  $self->{strip_tablename} = $opt[0] : $self->{strip_tablename};
 }
 
 sub values {
 	my $self 	= shift;
 	my @opt		= validate_pos(@_, {type => HASHREF, default => undef} );
-	return $opt[0] ?  $self->{values} = $opt[0] : $self->{values};
+	return defined $opt[0] ?  $self->{values} = $opt[0] : $self->{values};
 }
 
 sub tblschema {
 	my $self 	= shift;
 	my @opt		= validate_pos(@_, {isa => 'DBIx::DBSchema::Table', 
 									default => undef} );
-	return $opt[0] ?  $self->{tblschema} = $opt[0] : $self->{tblschema};
+	return defined $opt[0] ?  $self->{tblschema} = $opt[0] : $self->{tblschema};
 }
 
 sub labels {
 	my $self 	= shift;
 	my @opt		= validate_pos(@_,  {type => ARRAYREF | HASHREF,
 									default => undef} );
-	return $opt[0] ?  $self->{labels} = $opt[0] : $self->{labels};
+	return defined $opt[0] ?  $self->{labels} = $opt[0] : $self->{labels};
 	
 }
 
@@ -224,7 +252,7 @@ sub appearances {
 	my $self 	= shift;
 	my @opt		= validate_pos(@_,  {type => ARRAYREF | HASHREF,
 									default => undef} );
-	return $opt[0] ?  $self->{appearances} = $opt[0] : $self->{appearances};
+	return defined $opt[0] ?  $self->{appearances} = $opt[0] : $self->{appearances};
 	
 }
 
@@ -232,7 +260,47 @@ sub enums {
 	my $self 	= shift;
 	my @opt		= validate_pos(@_,  {type => ARRAYREF | HASHREF,
 									default => undef} );
-	return $opt[0] ?  $self->{enums} = $opt[0] : $self->{enums};
+	return defined $opt[0] ?  $self->{enums} = $opt[0] : $self->{enums};
+	
+}
+
+sub begin_form {
+	my $self 	= shift;
+	my @opt		= validate_pos(@_, {type => BOOLEAN, default => undef} );
+	return defined $opt[0] ?  $self->{begin_form} = $opt[0] : $self->{begin_form};
+}
+
+sub end_form {
+	my $self 	= shift;
+	my @opt		= validate_pos(@_, {type => BOOLEAN, default => undef} );
+	return defined $opt[0] ?  $self->{end_form} = $opt[0] : $self->{end_form};
+}
+sub cb_row_pre {
+	my $self	= shift;
+	my @opt		= validate_pos(@_,  {isa => 'Callback',
+									default => undef} );
+	return defined $opt[0] ?  $self->{cb_row_pre} = $opt[0] : $self->{cb_row_pre};
+	
+}
+sub cb_row_post {
+	my $self	= shift;
+	my @opt		= validate_pos(@_,  {isa => 'Callback',
+									default => undef} );
+	return defined $opt[0] ?  $self->{cb_row_post} = $opt[0] : $self->{cb_row_post};
+	
+}
+sub cb_rows_pre {
+	my $self	= shift;
+	my @opt		= validate_pos(@_,  {isa => 'Callback',
+									default => undef} );
+	return defined $opt[0] ?  $self->{cb_rows_pre} = $opt[0] : $self->{cb_rows_pre};
+	
+}
+sub cb_rows_post {
+	my $self	= shift;
+	my @opt		= validate_pos(@_,  {isa => 'Callback',
+									default => undef} );
+	return defined $opt[0] ?  $self->{cb_rows_post} = $opt[0] : $self->{cb_rows_post};
 	
 }
 
@@ -242,13 +310,17 @@ sub template() {
 									default => undef} );
 	$self->{template} = $opt[0] if ($opt[0]);
 	return $self->{template} || <<EOF;
-<TMPL_LOOP name="hidden_fields">
+<TMPL_IF form.begin>
+<FORM ACTION="" METHOD="POST">
+</TMPL_IF>
+%rows.pre%
+<TMPL_LOOP name="fields.hidden">
 	<input type="HIDDEN" name="%name%" value="%value%">
 </TMPL_LOOP>
 <table>
 <TMPL_LOOP name="fields">
 <tr>
-<td><b>%label%:</b></td>
+<td>%row.pre%<b>%label%:</b></td>
 <td>
 <TMPL_UNLESS name="use_label">
 <TMPL_IF name="can_be_null">
@@ -279,10 +351,16 @@ sub template() {
 <TMPL_IF name="use_unmodificable">
 	%value%
 </TMPL_IF>
+%row.post%
 </td>
 </tr>
 </TMPL_LOOP>
+<TMPL_VAR name="rows.post">
 </table>
+<TMPL_IF form.end>
+<INPUT TYPE="submit" NAME="submit_form" VALUE="Invia">
+</FORM>
+</TMPL_IF>
 
 <script type="text/javascript">
 <!--
